package com.supervision.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.supervision.dto.roundAsk.SessionParamDTO;
import com.supervision.exception.BusinessException;
import com.supervision.service.AskService;
import com.supervision.vo.*;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import static com.supervision.service.impl.AskServiceImpl.SESSION_PARAM;

@Slf4j
@RestController
@RequestMapping("ask")
@RequiredArgsConstructor
public class AskController {

    private final AskService askService;

    private final RedisTemplate<String, Object> redisTemplate;

    @ApiOperation("多轮对话")
    @PostMapping("roundTalk")
    public RoundTalkResVO roundTalk(@RequestBody RoundTalkReqVO roundTalkReqVO) {
        if (StrUtil.isBlank(roundTalkReqVO.getSessionId())) {
            throw new BusinessException("会话ID不能为空");
        }
        try {
            return askService.roundTalk(roundTalkReqVO);
        } catch (Exception e) {
            log.info("多轮对话过程中出现了异常,则说明多轮对话走不通,那么就根据sessionId获取Redis中的原始问题,直接走单轮对话");
            // 如果多轮对话过程中出现了异常,则说明多轮对话走不通,那么就根据sessionId获取Redis中的原始问题,直接走单轮对话
            Object cache = redisTemplate.opsForValue().get(SESSION_PARAM + roundTalkReqVO.getSessionId());
            SessionParamDTO sessionParamDTO = BeanUtil.toBean(cache, SessionParamDTO.class);
            SingleTalkResVO singleTalkResVO = askService.singleTalk(SingleTalkReqVO.builder().userTalk(sessionParamDTO.getOriginalQuestion()).build());
            return RoundTalkResVO.builder().sessionId(roundTalkReqVO.getSessionId())
                    .answerText(singleTalkResVO.getAnswerText()).build();
        }

    }

    @ApiOperation("单轮对话")
    @PostMapping("singleTalk")
    public SingleTalkResVO singleTalk(@RequestBody SingleTalkReqVO singleTalkReqVO) {
        log.info("单轮对话问题开始:{}", singleTalkReqVO.getUserTalk());
        SingleTalkResVO singleTalkResVO = askService.singleTalk(singleTalkReqVO);
        log.info("单轮对话问题结束:{}", singleTalkReqVO.getUserTalk());
        return singleTalkResVO;
    }

    @ApiOperation("多轮对话中用户手动填写参数,可能直接返回结果")
    @PostMapping("saveUserParam")
    public RoundTalkResVO saveUserParam(@RequestBody RoleSetReqVO paramReqVO) {
        return askService.saveUserParam(paramReqVO);
    }

    @ApiOperation("查询多轮对话中用户需要填写的参数")
    @GetMapping("queryUserNeedParam")
    public RoleSetResVO queryUserNeedParam(String sessionId) {
        return askService.queryUserNeedParam(sessionId);
    }
}
